# RaphGraph
# Version 1.0
# Relies on jQuery 1.4.2+
# Developed by Lexer
window.RaphGraph = {}

# These styles are assigned to each graph by default.
# You can easily override the globaly styles by overriding
# this object on the window. Each graph also accepts a
# style option, which will override this stylesheet for
# that specific instance.
window.RaphGraph.DefaultStyle = {
  # dimensions should be supplied on a per graph basis
  width: null,
  height: null,
  
  # setting barHeight will make the height of the graph dynamic
  barHeight: null,
  
  # colors are intentionally ugly ;P
  # fill color can accept an array, which will cycle throughout for each entry
  fillColor: "#000000",
  gridColor: "#eeeeee",
  backgroundColor: "#ffffff",
  
  # textured graphs look for files named by color in the specified image folder
  textured: false,
  texturedURL: "/images"
  
  # if no border color is set, the fill will be applied
  borderColor: null,
  
  # if border == 0 then no border will be drawn
  border: 0,
  
  # graph padding is the distance of the graph from the edge of the canvas
  graphPadding: 10,
  
  # bar padding is the distance between each bar
  barPadding: 5,
  
  # label padding is the spacing between the label and edges of the bar
  labelPadding: 5,
  legendPadding: 50,
  
  dotRadius: 3,
  
  # fonts start at maxSize and shrink to minSize to fit.
  fontColor: "#ffffff",
  fontFamily: "Verdana",
  fontMax: 10,
  fontMin: 3,
  
  # how rounded would you like those corners?
  cornerRadius: 2,
  
  # animation speed (in milliseconds)
  animationSpeed: 250
}

window.RaphGraph.Language = {
  error: "There was an error"
}

# Base class for all graphs
class window.RaphGraph.Base
  colorIndex: null
  isLog: false
  renderLegend: false
  style: $.extend( true, {}, window.RaphGraph.DefaultStyle )
  constructor: (options) ->
    # set the options
    @setOption(tKey, tValue) for tKey, tValue of (options||{})
    
    # setup the canvas
    @drawCanvas()
    
    @element.trigger "ready"
    
    # get the data
    if @data?
      @fetchData()
  
  
  setOption: (tKey, tValue) ->
    if tKey is 'element'
      @element = $(tValue)
    
    if tKey is 'data'
      @data = tValue
    
    if tKey is 'click'
      @click = tValue
    
    if tKey is 'style'
      @style = $.extend true, {}, @style, tValue
    
    if tKey is 'log'
      @isLog = tValue
    
    if tKey is 'legend'
      @renderLegend = tValue
    
  
  
  # does math on the value if a particular form of scaling has been applied
  scale: (tValue) ->
    if @isLog
      Math.log tValue
    else
      tValue
  
  
  # sets up the initial canvas
  drawCanvas: ->
    # empty the element
    @element.html ""
    
    # save the original dimensions for later
    @_originalDimensions = {
      width:   @width || @element.width(),
      height: @height || @element.height()
    }
    
    # setup the canvas
    @canvas = Raphael( @element[0], @_originalDimensions.width, @_originalDimensions.height )
    
    # draw the background
    backgroundDimensions = {}
    if ( @_originalDimensions.width-1 < 0)
      backgroundDimensions.width = @_originalDimensions.width
    else
      backgroundDimensions.width = @_originalDimensions.width-1
    
    if ( @_originalDimensions.height-1 < 0)
      backgroundDimensions.height = @_originalDimensions.height
    else
      backgroundDimensions.height = @_originalDimensions.height-1
    
    @background = @canvas.rect( 0, 0, backgroundDimensions.width, backgroundDimensions.height )
    @background.attr {
      fill: @style.backgroundColor,
      stroke: "none"
    }
    @background.toBack()
  
  
  # draws a loader on the canvas on initial request
  showLoader: ->
    @loader = new window.RaphGraph.Loader @canvas, @style
  
  
  # hides the loader
  hideLoader: ->
    @loader.remove()
  
  
  # draws an error message on the canvas on initial request
  showError: ->
    @error = new window.RaphGraph.Error @canvas, @style
  
  
  # hides the error message
  hideError: ->
    if @error?
      @error.remove()
  
  
  # makes the request to load the data from the server
  fetchData: (tURL, tData) ->
    # trigger the begin load event
    @element.trigger "load", [tURL, tData]
    
    # show the loader
    @hideError()
    @showLoader()
    
    # make the request
    $.ajax({
      url: tURL||@data,
      data: tData||{},
      success: (data, status, xmlhttprequest) =>
        if xmlhttprequest.status is 200
          @processResponse data
        else
          @processEmptyResponse()
      error: (tError) =>
        @responseError tError
    })
  
  
  # on ajax response, this guy processes the data
  processResponse: (tResponse) ->
    # trigger the begin load event
    @element.trigger "success", [tResponse]
    
    # set the response data
    @graphData = tResponse
    
    # draw the graph
    @drawGraph()
  
  
  # on an empty ajax response
  processEmptyResponse: ->
    # trigger the begin load event
    @element.trigger "success", [[]]
    
    # set the response data
    @graphData = []
    
    # remove the loader
    @hideLoader()
    
    # show the error message
    @showError()
  
  
  # on error response, this guy makes things happen
  responseError: (tError) ->
    # trigger the begin load event
    @element.trigger "error", tError
    
    # remove the loader
    @hideLoader()
    
    # show the error message
    @showError()
  
  
  # designed to be overridden, this is where the drawing happens
  # should called at the end of the drawing execution
  drawGraph: ->
    @hideLoader()
    
    # if the height is dynamic, get the contents height and set the canvas
    if @style.barHeight? and @_finalHeight?
      @canvas.setSize @_originalDimensions.width, @_finalHeight
      @background.attr {
        width: @_originalDimensions.width-1,
        height: @_finalHeight-1 }
    else
      @canvas.setSize @_originalDimensions.width, @_originalDimensions.height
      @background.attr {
        width: @_originalDimensions.width-1,
        height: @_originalDimensions.height-1 }
    
    
    @background.attr {
      fill: @style.backgroundColor,
      stroke: "none"
    }
    @background.toBack()
    
    @element.trigger "drawn"
  
  
  # removes all elements from the graph
  resetGraph: ->
    @canvas.clear()
    
    # draw the background
    @background = @canvas.rect( 0, 0, @canvas.width-1, @canvas.height-1 )
    @background.attr {
      fill: @style.backgroundColor,
      stroke: "none"
    }
    @background.toBack()
  
  
  # cleans the label for presentation
  cleanLabel: (label) ->
    label = label.split("_").join(" ")
    label = label.toLowerCase()
    
    newLabel = ""
    for word in label.split(" ")
      if newLabel.length
        newLabel = "#{newLabel} #{word.substring(0,1).toUpperCase()}#{word.substring(1,word.length)}"
      else
        newLabel = "#{word.substring(0,1).toUpperCase()}#{word.substring(1,word.length)}"
    
    newLabel
  
  
  # returns a color and increments the colour counter
  getColor: ->
    # do we have an array of colors?
    if @style.fillColor instanceof Array
      
      # do we have a color index yet?
      if !@colorIndex?
        @colorIndex = Math.floor(Math.random() * @style.fillColor.length)
      
      # choose a color
      color = @style.fillColor[@colorIndex]
      
      # calculate the color range
      @colorIndex ++
      if @colorIndex+1 > @style.fillColor.length
        @colorIndex = 0
      
      color
      
    else
      @style.fillColor
  
  
  fillForColor: (color) ->
    if @style.textured
      color = color.split("#")[1]
      "url(#{@style.texturedURL}/pattern-#{color}.png)"
    else
      color
  
  
  shrinkLabelToFitInRect: (label, rect) ->
    # make the label fit
    for size in [@style.fontMax..(@style.fontMin-1)] by -1
      if label.getBBox().width < (rect.getBBox().width - (@style.labelPadding * 2))
        break
      if size < @style.fontMin
        label.remove()
        break
      label.attr {
        "font-size":size
      }
    
  
  
  actionPanelForRect: (rect, click, labelData, rawData) ->
    # draw the rect
    hoverRect = rect.paper.rect rect.getBBox().x, rect.getBBox().y, rect.getBBox().width, rect.getBBox().height
    hoverRect.attr {
      fill: "#f00",
      "fill-opacity": 0.0,
      stroke: "none",
    }
    
    # setup the panel events
    ((rect, clickFunction, data) ->
      $(rect.node).mouseover () ->
        rect.panel = new window.RaphGraph.CenteredPanel rect.paper, data
      $(rect.node).mouseout () ->
        rect.panel.remove()
      $(rect.node).click () ->
        clickFunction(rawData)
    )(hoverRect, click||`function(){}`, labelData, rawData)
    
    hoverRect
  



class window.RaphGraph.Loader
  style: window.RaphGraph.DefaultStyle
  shouldFinish: false
  constructor: (tCanvas, tStyle) ->
    width = 10
    @shouldFinish = false
    
    $.extend(true, @style, (tStyle||{}))
    
    x1 = (tCanvas.width/2)-(width*1.25)
    x2 = (tCanvas.width/2)+(width*0.25)
    
    y = (tCanvas.height/2)-(width*0.5)
    
    style = {
      fill: @style.gridColor,
      stroke: "none"
    }
    
    @blockA = tCanvas.rect(x1, y, width, width)
    @blockA.attr style
    
    @blockB = tCanvas.rect(x2, y, width, width)
    @blockB.attr style
    
    @animatePhaseOne()
    
    @
  
  remove: ->
    @shouldFinish = true
    @blockA.remove()
    @blockB.remove()
  
  animatePhaseOne: ->
    @blockA.animate {opacity:1.0}, @style.animationSpeed
    @blockB.animate {opacity:0.0}, @style.animationSpeed, () =>
      @animatePhaseTwo()
  
  animatePhaseTwo: ->
    @blockA.animate {opacity:0.0}, @style.animationSpeed
    @blockB.animate {opacity:1.0}, @style.animationSpeed, () =>
      @animatePhaseOne()
  


class window.RaphGraph.Error
  style: window.RaphGraph.DefaultStyle
  constructor: (tCanvas, tStyle) ->
    $.extend(true, @style, (tStyle||{}))
    
    # make the label
    @text = tCanvas.text((tCanvas.width/2), (tCanvas.height/2), window.RaphGraph.Language.error)
    @text.attr {
      "fill": @style.fontColor,
      "font-family": "Verdana",
      "font-size": @style.fontMax,
      opacity: 0.0
    }
    
    # draw the box behind it
    @block = tCanvas.rect(@text.getBBox().x - 10, @text.getBBox().y - 10, @text.getBBox().width + 20, @text.getBBox().height + 20, @style.cornerRadius)
    @block.attr {
      fill: @style.gridColor,
      stroke: "none",
      opacity: 0.0
    }
    
    @text.toFront()
    @fadeIn @block
    
    @
  
  remove: ->
    @fadeOut()
  
  fadeIn: ->
    @block.animate {opacity:1.0}, @style.animationSpeed
    @text.animate {opacity:1.0}, @style.animationSpeed
  
  fadeOut: ->
    @text.animate {opacity:0.0}, @style.animationSpeed, () =>
      @text.remove
    @block.animate {opacity:0.0}, @style.animationSpeed, () =>
      @block.remove
  



class window.RaphGraph.CenteredPanel
  style: $.extend(true, {}, window.RaphGraph.DefaultStyle)
  constructor: (tCanvas, tData) ->
    @panel = tCanvas.set()
    
    # generate the text
    labelText = ""
    for tLabel, tValue of tData
      labelText = "#{labelText}#{tLabel}: #{tValue}\n"
    
    # set the text
    x     = (tCanvas.width)/2
    y     = (tCanvas.height)/2
    label = tCanvas.text x, y, labelText
    label.attr {
      fill: @style.fontColor,
      "font-size": @style.fontMax,
      "font-family": @style.fontFamily,
      stroke:"none" }
    @panel.push label
    
    # draw the box
    x      = label.getBBox().x - @style.graphPadding
    y      = label.getBBox().y - @style.graphPadding
    width  = label.getBBox().width + (@style.graphPadding*2)
    height = label.getBBox().height + (@style.graphPadding*2)
    rect   = tCanvas.rect x, y, width, height, @style.cornerRadius
    rect.attr {fill:"#000", stroke:"none", opacity:0.0}
    @panel.push rect
    
    label.toFront()
    
    # and.... REVEAL!!!!
    @panel.attr {opacity:0.0}
    label.animate {opacity:1.0}, @style.animationSpeed
    rect.animate {opacity:0.75}, @style.animationSpeed
  
  remove: ->
    @panel.animate {opacity:0.0}, @style.animationSpeed, () =>
      @panel.remove
  


class window.RaphGraph.DetailedPanel
  style: $.extend(true, {}, window.RaphGraph.DefaultStyle)
  constructor: (paper, graphData, dataIndex, x, y) ->
    @panel = paper.set()
    
    # generate the text
    labelText = ""
    for tGroup, tData of graphData
      labelText = "#{labelText}#{tGroup}: #{graphData[tGroup][dataIndex]['value']}\n"
    labelText = "#{graphData[tGroup][dataIndex]['label']}\n#{labelText}"
    
    # calculate the labels position
    x = x + @style.graphPadding || paper.width  / 2
    y = y + @style.graphPadding || paper.height / 2
    
    # create the label
    label = paper.text x, y, labelText
    label.attr {
      fill: @style.fontColor,
      "font-size": @style.fontMax,
      "font-family": @style.fontFamily,
      stroke:"none" }
    @panel.push label
    
    # draw the box
    width  = label.getBBox().width  + ( @style.graphPadding * 2 )
    height = label.getBBox().height + ( @style.graphPadding * 2 )
    
    # make sure the box is inside the graph area
    if (x + width) > paper.width
      x = paper.width - @style.graphPadding - width
    
    rect = paper.rect x, y, width, height, @style.cornerRadius
    rect.attr {
      fill:"#000",
      stroke:"none",
      opacity:0.0}
    @panel.push rect
    
    # revise the position of the label should be in the middle of the rect
    label.attr {
      x: x + (width  / 2),
      y: y + (height / 2)
    }
    
    # the label should be in front of the background
    label.toFront()
    
    # and.... REVEAL!!!!
    @panel.attr   {opacity: 0.0}
    label.animate {opacity: 1.0}, @style.animationSpeed
    rect.animate  {opacity:0.75}, @style.animationSpeed
    
    @
  
  remove: ->
    @panel.animate {opacity:0.0}, @style.animationSpeed, () =>
      @panel.remove
    
  



class window.RaphGraph.PieGraph extends window.RaphGraph.Base
  drawGraph: ->
    # calculate the total sum
    totalSum =  0
    totalSum += @graphData[tItem]['value'] for tItem of @graphData
    
    # what's the render width
    renderWidth  =  @canvas.width - ( @style.graphPadding * 2 ) + @style.barPadding
    renderHeight = @canvas.height - ( @style.graphPadding * 2 )
    
    # set the width controller
    widthProgress = 0
    
    # create a set
    graphSet = @canvas.set()
    textSet  = @canvas.set()
    hoverSet = @canvas.set()
    
    # draw the items
    for tItem of @graphData
      # get the entries data
      tKey   = @graphData[tItem]['label']
      tValue = @graphData[tItem]['value']
      
      # calculate the items width
      rectWidth = Math.max( Math.floor( ( tValue / totalSum ) * renderWidth ) - @style.barPadding - (@style.border), 0 )
      
      # draw the rect
      rect = @canvas.rect @style.graphPadding + widthProgress, @style.graphPadding, rectWidth, renderHeight, @style.cornerRadius
      fillColor = @getColor()
      rect.attr {
        opacity: 0.0,
        fill: @fillForColor(fillColor),
        stroke: @style.borderColor||fillColor,
        "stroke-width": @style.border
      }
      graphSet.push rect
      
      # draw the label
      labelText = @cleanLabel tKey
      labelX = @style.graphPadding + widthProgress + ( rectWidth / 2 )
      labelY = ( renderHeight / 2 ) + @style.graphPadding
      label  = @canvas.text labelX, labelY, labelText
      label.attr {
        opacity: 0.0,
        "fill": @style.fontColor,
        "font-family": @style.fontFamily,
        "font-size": @style.fontMax
      }
      textSet.push label
      @shrinkLabelToFitInRect label, rect
      
      # create a hover rect
      hoverSet.push @actionPanelForRect rect, @click, {"Type":@cleanLabel(tKey), "Frequency":tValue}, @graphData[tItem]
      
      # recalculate the width
      widthProgress += rectWidth + @style.barPadding + (@style.border)
    
    
    # animate the rect
    graphSet.animate {opacity: 1.0}, @style.animationSpeed
    textSet.animate  {opacity: 1.0}, @style.animationSpeed
    
    hoverSet.toFront()
    
    super()
  


class window.RaphGraph.BarGraph extends window.RaphGraph.Base
  drawGraph: ->
    # get the min & max values
    minValue = Infinity
    maxValue = 0
    for tIndex, tData of @graphData
      if tData['value'] > maxValue
        maxValue = tData['value']
      if tData['value'] < minValue
        minValue = tData['value']
    
    # make sure the two values aren't the same
    if minValue is maxValue
      # if all the rows are the same, we can't present in log
      @isLog = false
      # min value is 0
      minValue = 0
    
    # calculate the scale
    maxValue = @scale maxValue
    minValue = @scale minValue
    
    # what's the render width & height
    renderWidth  =  @canvas.width - ( @style.graphPadding * 2 )
    renderHeight = @canvas.height - ( @style.graphPadding * 2 )
    
    # did the user specify a bar height?
    if @style.barHeight?
      barHeight = @style.barHeight - @style.border
    else
      barHeight = Math.floor( ( renderHeight - ( ( @graphData.length - 1 ) * @style.barPadding ) ) / @graphData.length ) - (@style.border)
    
    # create a set
    backgroundSet = @canvas.set()
    graphSet      = @canvas.set()
    textSet       = @canvas.set()
    hoverSet      = @canvas.set()
    
    # loop di loop
    heightProgress = 0
    for tItem of @graphData
      # get the entries data
      tKey        = @graphData[tItem]['label']
      tLabelValue = @graphData[tItem]['value']
      tValue      = @scale @graphData[tItem]['value']
      
      # calculate the items width
      rectWidth = Math.max( Math.floor( ( ( tValue - minValue ) / ( maxValue - minValue ) ) * renderWidth ), 10 )
      
      # draw the rect
      backgroundRect = @canvas.rect @style.graphPadding, @style.graphPadding + heightProgress, renderWidth, barHeight, @style.cornerRadius
      rect           = @canvas.rect @style.graphPadding, @style.graphPadding + heightProgress,   rectWidth, barHeight, @style.cornerRadius
      
      # style the rect
      fillColor = @getColor()
      backgroundRect.attr {
        opacity: 0.0,
        fill:    @style.gridColor,
        stroke:  "none"
      }
      rect.attr {
        opacity: 0.0,
        fill:    @fillForColor(fillColor),
        stroke:  @style.borderColor||fillColor,
        "stroke-width": @style.border
      }
      backgroundSet.push backgroundRect
      graphSet.push rect
      
      # draw the label
      labelText = @cleanLabel tKey
      labelY    = @style.graphPadding + heightProgress + (barHeight / 2)
      label     = @canvas.text 0, labelY, labelText
      label.attr {
        opacity:       0.0,
        "fill":        @style.fontColor,
        "font-family": @style.fontFamily,
        "font-size":   @style.fontMax
      }
      textSet.push label
      
      # reposition the label into the proper place
      labelWidth = label.getBBox().width
      label.attr {
        x: @style.labelPadding + (labelWidth / 2) + @style.graphPadding
      }
      
      # if the label is wider than the graph, push it out of the graph
      if (labelWidth + (@style.labelPadding * 2 )) > rectWidth
        label.attr {
          fill: fillColor,
          x: rectWidth + @style.labelPadding + (labelWidth / 2) + @style.graphPadding
        }
      
      # if the label flows over the graph bounds, kill it
      if ((labelWidth/2) + label.getBBox().x - (@style.graphPadding*2)) > renderWidth
        label.remove()
      
      # create a hover rect
      hoverSet.push @actionPanelForRect backgroundRect, @click, {"Type":@cleanLabel(tKey), "Frequency":Math.floor(tLabelValue)}, @graphData[tItem]
      
      # recalculate the width
      heightProgress = heightProgress + barHeight + @style.barPadding + (@style.border)
    
    
    # animate the rect
    backgroundSet.animate {opacity: 1.0}, @style.animationSpeed
    graphSet.animate {opacity: 1.0}, @style.animationSpeed
    textSet.animate {opacity: 1.0}, @style.animationSpeed
    
    hoverSet.toFront()
    
    @_finalHeight = heightProgress + (@style.graphPadding*2) + @style.border
    
    super()
  


class window.RaphGraph.LineGraph extends window.RaphGraph.Base
  graphScale: "1"
  resetGraph: ->
    @graphScale = "1"
    super()
  
  drawGraph: ->
    # get the min & max values
    minValue = Infinity
    maxValue = 0
    for tGroup, tData of @graphData
      for tEntry in @graphData[tGroup]
        if tEntry['value'] > maxValue
          maxValue = tEntry['value']
        if tEntry['value'] < minValue
          minValue = tEntry['value']
    
    
    # if we've got zero results, draw a line at 10
    if maxValue is 0
      maxValue = 20
    
    
    # calculate the scale
    for tLetter in [0..String(maxValue).length-2]
      @graphScale = @graphScale + "0"
    @graphScale = Number(@graphScale)
    
    
    # make sure the values are divisible by 10
    minValue = Math.floor( minValue / @graphScale ) * @graphScale
    maxValue = Math.ceil(  maxValue / @graphScale ) * @graphScale
    
    # adjust the min and max by padding a fraction of the scale
    graphPadding = @graphScale / 4
    if (minValue-@graphScale) >= 0
      minValue = minValue - @graphScale
    maxValue = maxValue + graphPadding
    
    
    # renderable area
    renderWidth  = @canvas.width  - ( @style.graphPadding * 2 )
    renderHeight = @canvas.height - ( @style.graphPadding * 2 )
    
    # -------------------
    # draw the legend (if needed)
    legendHeight = 0
    if @renderLegend
      if !@colorIndex?
        @getColor()
      savedColor = @colorIndex
      
      longest = 0
      height  = 0
      count   = 0
      for tGroupName, tData of @graphData
        count  = count + 1
        tLabel = @canvas.text( 0, 0, tGroupName )
        
        tLabel.attr {
          "font-family": @style.fontFamily,
          "font-size": @style.fontMax
        }
        
        if tLabel.getBBox().width > longest
          longest = tLabel.getBBox().width
          height  = tLabel.getBBox().height
        
        tLabel.remove()
      
      columnWidth = longest + @style.legendPadding + (height / 2)
      columns     = Math.floor((@canvas.width - ( @style.graphPadding * 2 )) / columnWidth)
      
      rowHeight = height + @style.barPadding
      rows      = Math.ceil(count / columns)
      
      # this time we'll be drawing them
      column = 0
      row    = 0
      legendText  = @canvas.set()
      legendIcons = @canvas.set()
      for tGroupName, tData of @graphData
        tColor = @getColor()
        tX = (column * columnWidth) + @style.graphPadding
        tY = renderHeight - @style.graphPadding - (((rows-1)-row) * rowHeight)
        
        label =  @canvas.text 0, tY, tGroupName
        label.attr {
          x: ((column * columnWidth) + @style.graphPadding + (label.getBBox().width/2) + label.getBBox().height + (label.getBBox().height/2)),
          fill: tColor,
          "font-family": "#{@style.fontFamily}",
          "font-size": "#{@style.fontMax}" }
        legendText.push label
        
        box =  @canvas.rect (column * columnWidth) + @style.graphPadding, tY-(label.getBBox().height/2), label.getBBox().height, label.getBBox().height, @style.cornerRadius/2
        box.attr {
          fill: tColor,
          stroke: "none"
        }
        
        if column == columns
          row    = row + 1
          column = 0
        else
          column = column + 1
      
      legendHeight = (rowHeight * rows) + @style.graphPadding
      @colorIndex  = savedColor
    
    # -------------------
    # back to drawing the graph
    
    # calculate the grid size
    gridX = @style.graphPadding
    gridY = @style.graphPadding
    gridWidth  = @canvas.width  - ( @style.graphPadding * 2 )
    gridHeight = @canvas.height - ( @style.graphPadding * 2 ) - legendHeight
    
    # Draw the graph rect
    gridLines  = @canvas.set()
    gridLabels = @canvas.set()
    gridLines.push(
      @canvas.rect gridX+0.5, gridY, gridWidth, gridHeight-0.5
    )
    
    # calculate gridline spacing
    gridlineDistribution = ( (maxValue - minValue) - graphPadding ) / @graphScale
    gridlineSpacing = ( gridHeight / gridlineDistribution )
    if gridlineSpacing < 50
      gridlineSpacing = gridlineSpacing * Math.floor( 50 / gridlineSpacing )
      gridlineDistribution = Math.floor(gridHeight / gridlineSpacing)
    
    
    # draw the max value label
    label = @canvas.text 0, 0, "#{Math.floor(((gridlineDistribution)*@graphScale))}"
    textY = (@style.labelPadding*2) + @style.graphPadding
    textX = ( label.getBBox().width / 2 ) + @style.labelPadding + @style.graphPadding
    label.attr {
      x: textX,
      y: textY
    }
    gridLabels.push label
    
    # draw the gridlines
    for i in [1..gridlineDistribution]
      if gridlineDistribution != i
        pathString = "M#{@style.graphPadding} #{Math.floor(gridlineSpacing*i)+@style.graphPadding+0.5}L#{gridWidth+@style.graphPadding} #{Math.floor(gridlineSpacing*i)+@style.graphPadding+0.5}"
        path = @canvas.path pathString
      
      label = @canvas.text 0, 0, "#{Math.floor(((i-gridlineDistribution)*@graphScale)*(-1))}"
      textY = ( gridlineSpacing * i ) - (@style.labelPadding*2) + @style.graphPadding
      textX = ( label.getBBox().width / 2 ) + @style.labelPadding + @style.graphPadding
      label.attr {
        x: textX,
        y: textY
      }
      
      gridLines.push path
      gridLabels.push label
    
    
    # style the grid
    gridLines.attr {
       stroke: @style.gridColor,
      opacity: 0.0 }
    gridLabels.attr {
         fill: @style.gridColor,
      opacity: 0.0 }
    
    # create an all dots set so we can bring them all to front later
    allDots = @canvas.set()
    
    # Cycle through the groups
    for tGroup, tData of @graphData
      
      tEntry = @graphData[tGroup][0]
      y = gridHeight - ( ( tEntry['value'] / ( maxValue - graphPadding ) ) * gridHeight ) + @style.graphPadding
      pathString = "M#{@style.graphPadding},#{y}"
      dots = @canvas.set()
      
      # cycle through the entries
      for tIndex in [0..@graphData[tGroup].length-1]
        tEntry = @graphData[tGroup][tIndex]
        
        x =  ( ( tIndex / ( @graphData[tGroup].length - 1 ) ) * gridWidth ) + @style.graphPadding
        y = gridHeight - ( ( tEntry['value'] / ( maxValue - graphPadding ) ) * gridHeight ) + @style.graphPadding
        
        pathString = "#{pathString}L#{x},#{y}"
        dot = @canvas.circle x, y, @style.dotRadius
        dots.push dot
        
        # the fill is the same as the path, but with closure
        fillString = "#{pathString}L#{gridWidth+@style.graphPadding},#{gridHeight+@style.graphPadding}"
        fillString = "#{fillString}L#{@style.graphPadding},#{gridHeight+@style.graphPadding}z"
      
      
      # draw the path and the fill
      fill = @canvas.path fillString
      path = @canvas.path pathString
      
      # the dots should be above them all
      dots.toFront()
      allDots.push dots
      
      # get a color
      fillColor = @getColor()
      
      # style the three elements
      path.attr {
        stroke: @style.borderColor||fillColor,
        "stroke-width": @style.border,
        opacity: 1.0 }
      fill.attr {
        fill: @fillForColor(fillColor),
        "fill-opacity": 0.1,
        stroke: "none",
        opacity: 1.0 }
      dots.attr {
        fill: @style.borderColor||fillColor,
        stroke:"none",
        opacity: 1.0 }
      
      # animate the rect
      path.animate {opacity: 1.0}, @style.animationSpeed
      fill.animate {opacity: 1.0}, @style.animationSpeed
      dots.animate {opacity: 1.0}, @style.animationSpeed
    
    allDots.toFront()
    
    gridLines.animate  {opacity: 1.0}, @style.animationSpeed
    gridLabels.animate {opacity: 1.0}, @style.animationSpeed
    
    # create another set of rects for labels
    hoverRects = @canvas.set()
    for tGroup, tData of @graphData
      for tIndex in [0..@graphData[tGroup].length-1]
        width  = gridWidth / ( @graphData[tGroup].length - 1 )
        x      = ( ( tIndex / ( @graphData[tGroup].length - 1 ) ) * gridWidth ) + @style.graphPadding - ( width / 2 )
        y      = @style.graphPadding
        height = gridHeight
        
        rect   = @canvas.rect x, y, width, height
        rect.attr {
          fill: "#000",
          "fill-opacity": 0.0,
          stroke: "none" }
        
        hoverRects.push rect
        
        # setup the panel events
        ((rect, hoverRects, graphData, tIndex, x, y) ->
          $(rect.node).mouseover () ->
            rect.panel = new window.RaphGraph.DetailedPanel rect.paper, graphData, tIndex, x, y
            hoverRects.toFront()
          $(rect.node).mouseout () ->
            rect.panel.remove()
        )(rect, hoverRects, @graphData, tIndex, x+width, y)
        
      break
    
    
    super()
  


class window.RaphGraph.CloudGraph extends window.RaphGraph.Base
  drawGraph: ->
    # what's the max value?
    maxValue = 0
    minValue = Infinity
    for tIndex, tData of @graphData
      if tData['value'] > maxValue
        maxValue = tData['value']
      if tData['value'] < minValue
        minValue = tData['value']
    
    if minValue == maxValue
      minValue = 0
    
    # calculate the scale
    maxValue = @scale maxValue
    minValue = @scale minValue
    
    # create a bar array
    bars = []
    
    # for each bar 100% is set to the highest item value
    valueWidth = maxValue * 1.75
    
    # loop through the values and build the bar array
    currentBar      = 0
    currentBarWidth = 0
    while !@allDataHasBeenAdded()
      # create the bar if it doesn't exist
      if !bars[currentBar]?
        bars[currentBar] = {
          bars: []
          value: 0
        }
      
      # loop through the unprocessed bars
      for tIndex, tData of @graphData
        if !tData['processed']?
          # if it fits
          tValue = @scale(tData['value'])
          if (tValue + currentBarWidth) <= valueWidth
            # add it to the current bar
            tData['processed'] = true
            currentBarWidth    = currentBarWidth + tValue
            bars[currentBar].bars.push tData
            bars[currentBar].value =  currentBarWidth
      
      currentBar = currentBar + 1
      currentBarWidth = 0
    
    
    # what's the render width & height
    renderWidth  =  @canvas.width - ( @style.graphPadding * 2 )
    renderHeight = @canvas.height - ( @style.graphPadding * 2 )
    
    # what's the height of each row going to be?
    if @style.barHeight?
      barHeight = @style.barHeight - @style.border
    else
      barHeight = Math.floor( ( renderHeight - ( ( bars.length - 1 ) * @style.barPadding ) ) / bars.length ) - @style.border
    
    # what's the allowed row variance
    allowedVariance = maxValue * 0.25
    
    # create a set
    backgroundSet = @canvas.set()
    graphSet =      @canvas.set()
    textSet =       @canvas.set()
    hoverSet =      @canvas.set()
    
    # loop through the bars
    for tIndex, tBar of bars
      widthProgress = 0
      
      # if the bars value is much less than the maxValue,
      # we whould render it at the correct scale
      tMaxValue = tBar.value
      if tMaxValue < (maxValue - allowedVariance)
        barWidth = renderWidth * (tMaxValue / (maxValue - allowedVariance))
      else
        barWidth = renderWidth
      
      
      barY = @style.graphPadding + (tIndex * (barHeight + @style.barPadding + @style.border))
      
      # draw the background bar
      backgroundRect = @canvas.rect @style.graphPadding, barY, renderWidth, barHeight, @style.cornerRadius
      backgroundRect.attr {
        opacity: 1.0,
        fill: @style.gridColor,
        stroke: "none"
      }
      backgroundSet.push backgroundRect
      
      for tBarIndex, tBarItem of tBar.bars
        # get the entries data
        tLabel = tBarItem['label']
        tOriginalValue = tBarItem['value']
        tValue = @scale(tBarItem['value'])
        
        # calculate the items width
        rectWidth = ( (tValue / tMaxValue) * (barWidth + @style.barPadding) ) - @style.border - @style.barPadding
        
        # draw the main rect
        rect = @canvas.rect @style.graphPadding + widthProgress + (tBarIndex*@style.barPadding), barY, rectWidth, barHeight, @style.cornerRadius
        fillColor = @getColor()
        rect.attr {
          opacity: 0.0,
          fill: @fillForColor(fillColor),
          stroke: @style.borderColor||fillColor,
          "stroke-width": @style.border }
        graphSet.push rect
        
        # draw the label
        labelText = @cleanLabel tLabel
        labelX    = @style.graphPadding + widthProgress + ( rectWidth / 2 ) + (tBarIndex*@style.barPadding)
        labelY    = ( barHeight / 2 ) + barY
        label     = @canvas.text labelX, labelY, labelText
        label.attr {
          "fill": @style.fontColor,
          "font-family": @style.fontFamily,
          "font-size": @style.fontMax
        }
        textSet.push label
        @shrinkLabelToFitInRect label, rect
        
        # draw the hover rect
        hoverSet.push = @actionPanelForRect rect, @click, {"Tag":@cleanLabel(tLabel), "Frequency":tOriginalValue}, tBarItem
        
        widthProgress = widthProgress + rectWidth + (@style.border)
    
    
    # animate the rect
    graphSet.animate {opacity: 1.0}, @style.animationSpeed
    textSet.animate  {opacity: 1.0}, @style.animationSpeed
    
    hoverSet.toFront()
    
    @_finalHeight = (@style.graphPadding * 2) + ((barHeight + @style.barPadding) * bars.length) + @style.border
    
    super
  
  
  allDataHasBeenAdded: ->
    itemCount  = @graphData.length
    addedCount = 0
    for tIndex, tData of @graphData
      if tData['processed']?
        addedCount = addedCount + 1
    
    if itemCount == addedCount
      true
    else
      false
    
  
